-- Throttle
--
-- Version 2.1
-- Autor Ralf08
-- 
-- Change log:
-- 10.01.2016 / Ralf08 / Release
-- 06.11.2017 / Ralf08 / new on/off function MP added / FS17
-- 25.12.2017 / Ralf08 / more realistic min speed added / FS17



--local FirstRun
Throttle = {}
Throttle.enable = false


function Throttle.prerequisitesPresent(specializations)
	return true
end


function Throttle.registerEventListeners(vehicleType)
	for i, v in ipairs({"onLoad","onUpdate","onRegisterActionEvents"}) do
		SpecializationUtil.registerEventListener(vehicleType, v, Throttle)
	end
end


function Throttle:onLoad()
	local spec = self.spec_Throttle
    spec.changeDelay = 250
    spec.changeCurrentDelay = 0
    spec.speedSent = 0
	spec.vgetold = 0
	spec.elide = true
	spec.axisThrottle = 0
	
	if self.spec_drivable.cruiseControl.maxSpeed > 20 then
		spec.vreduced = 20
	else
		spec.vreduced = self.spec_drivable.cruiseControl.maxSpeed
	end
	spec.vset = self.spec_drivable.cruiseControl.maxSpeed
	
	
end


function Throttle:onUpdate(dt)
	local spec = self.spec_Throttle
	if self.isClient then
		if self.getIsEntered ~= nil and self:getIsEntered() then
			if self:getIsVehicleControlledByPlayer() and  self.spec_drivable.cruiseControl.isActive then
				if Throttle.enable and self.spec_motorized.isMotorStarted then
					if spec.axisThrottle == 0 and spec.elide then -- problem on farm silo elide
						spec.elide = false
					else
						spec.vget = math.floor(spec.vset * spec.axisThrottle)
						if spec.vget < self.spec_drivable.cruiseControl.minSpeed then
							spec.vget = 0
							if self.spec_drivable.cruiseControl.state == Drivable.CRUISECONTROL_STATE_ACTIVE then
								if self.spec_drivable.cruiseControl.speed == self.spec_drivable.cruiseControl.minSpeed and self:getLastSpeed() < self.spec_drivable.cruiseControl.minSpeed + 1 then
									self:setCruiseControlState(Drivable.CRUISECONTROL_STATE_OFF)
								end
							end
						elseif spec.vget >= spec.vset then
							spec.vget = spec.vset
						end
						if spec.vget ~= self.spec_drivable.cruiseControl.speed and not (spec.vget == 0 and self.spec_drivable.cruiseControl.speed == self.spec_drivable.cruiseControl.minSpeed) then
							spec.changeCurrentDelay = spec.changeCurrentDelay - (dt*10)
							if spec.changeCurrentDelay < 0 then
								spec.changeCurrentDelay = spec.changeDelay
								if spec.vget > self.spec_drivable.cruiseControl.speed then
									spec.setspeed = spec.vget
								elseif spec.vget < self.spec_drivable.cruiseControl.speed and self.spec_drivable.cruiseControl.speed > self.spec_drivable.cruiseControl.minSpeed then
									if self:getLastSpeed() - self.spec_drivable.cruiseControl.speed < -10 then
										if self:getLastSpeed() < spec.vget then
											spec.setspeed = spec.vget
										else
											spec.setspeed = self:getLastSpeed() - 1
										end									
									else
										spec.setspeed = self.spec_drivable.cruiseControl.speed - 1
									end
								else
									spec.setspeed = self.spec_drivable.cruiseControl.speed
								end
								if self.spec_drivable.cruiseControl.state == Drivable.CRUISECONTROL_STATE_OFF and spec.vgetold ~= spec.vget then
									self:setCruiseControlState(Drivable.CRUISECONTROL_STATE_ACTIVE)
								end
								spec.wasSpeedChanged = true
								self:setCruiseControlMaxSpeed(spec.setspeed)
							end
							spec.vgetold = spec.vget
						else
							spec.wasSpeedChanged = false
							spec.changeCurrentDelay = 0
						end
						spec.elide = true
					end

			
					if not spec.wasSpeedChanged and self.spec_drivable.cruiseControl.speed ~= spec.speedSent then
						if g_server ~= nil then
							g_server:broadcastEvent(SetCruiseControlSpeedEvent:new(self, self.spec_drivable.cruiseControl.speed), nil, nil, self)
						else
							g_client:getServerConnection():sendEvent(SetCruiseControlSpeedEvent:new(self, self.spec_drivable.cruiseControl.speed))
						end
						spec.speedSent = self.spec_drivable.cruiseControl.speed
					end
				end
			end
		end
	end
end


function Throttle:onRegisterActionEvents(isSelected, isOnActiveVehicle)
    local spec = self.spec_Throttle
	if self.isClient then
		self:clearActionEventsTable(spec.actionEvents)
		if  self:getIsActiveForInput(true) and not self:getIsAIActive() then
			local actionEventId
			_, actionEventId = self:addActionEvent(spec.actionEvents, InputAction.THROTTLE_TOGGLE, self, Throttle.Toggle, false, true, false, true, nil)
			g_inputBinding:setActionEventTextPriority(actionEventId, GS_PRIO_LOW)
			
			_, actionEventId = self:addActionEvent(spec.actionEvents, InputAction.THROTTLE_MODUS, self, Throttle.Modus, false, true, false, true, nil)
			g_inputBinding:setActionEventTextPriority(actionEventId, GS_PRIO_LOW)
			
            _, actionEventId = self:addActionEvent(spec.actionEvents, InputAction.THROTTLE_AXIS_PLUS_MINUS, self, Throttle.actionEventThrottle, false, false, true, true, nil)
            g_inputBinding:setActionEventTextPriority(actionEventId, GS_PRIO_VERY_LOW)
			g_inputBinding:setActionEventTextVisibility(actionEventId, false)

        end
	end
end


function Throttle.Toggle(self)
	Throttle.enable = not Throttle.enable
end


function Throttle.Modus(self)
	local spec = self.spec_Throttle
	if spec.vset == self.spec_drivable.cruiseControl.maxSpeed then
		spec.vset = spec.vreduced
	else
		spec.vset = self.spec_drivable.cruiseControl.maxSpeed
	end
end


function Throttle.actionEventThrottle(self, actionName, inputValue, callbackState, isAnalog)
    local spec = self.spec_Throttle
    spec.axisThrottle = MathUtil.clamp(inputValue, 0, 1)
end
